module NaspaySender
  class SenderBase
    def self.call(*args)
      new(*args).call
    end
  end

  class Transaction < SenderBase
    def initialize(intent, amount, currency, id, description, customer = nil, shipping_address = nil)
      secrets       = NaspayRails.configuration.secrets
      @api_endpoint = secrets[:api_endpoint]
      @auth_token   = NaspayRails::Auth.get_token(secrets)
      @params       = {
          intent:         intent,
          amount:         amount,
          currency:       currency,
          transaction_id: id,
          description:    description
      }
      if customer.present?
        @params.store(:customer, customer)
      end
      if shipping_address.present?
        @params.store(:shipping_address, shipping_address)
      end
    end

    def call
      NaspayRails::Transactions.create_new(@api_endpoint, @params, @auth_token)
    end
  end

  class Payout < SenderBase
    def initialize(amount, currency, id, description, card)
      secrets       = NaspayRails.configuration.secrets
      @api_endpoint = secrets[:api_endpoint]
      @auth_token   = NaspayRails::Auth.get_token(secrets)
      @params       = {
          amount:         amount,
          currency:       currency,
          transaction_id: id,
          description:    description,
          card:           card
      }
    end

    def call
      NaspayRails::Payouts.create_new(@api_endpoint, @params, @auth_token)
    end
  end
end
